package com.sun.beans.finder;

import java.util.HashMap;
import java.lang.reflect.Modifier;
import java.lang.reflect.Executable;

abstract class AbstractFinder<T extends Executable>
{
    private final Class<?>[] args;
    
    protected AbstractFinder(Class<?>[] args) {
        this.args = args;
    }
    
    protected boolean isValid(T t) {
        return Modifier.isPublic(t.getModifiers());
    }
    
    final T find(T[] array) throws NoSuchMethodException {
        HashMap<T, Class> hashMap = new HashMap();
        T t = null;
        Class[] array2 = null;
        boolean n = false;
        for (T t2 : array) {
            if (this.isValid(t2)) {
                Class[] parameterTypes = t2.getParameterTypes();
                if (parameterTypes.length == this.args.length) {
                    PrimitiveWrapperMap.replacePrimitivesWithWrappers(parameterTypes);
                    if (this.isAssignable(parameterTypes, this.args)) {
                        if (t == null) {
                            t = t2;
                            array2 = parameterTypes;
                        }
                        else {
                            int assignable = this.isAssignable(array2, parameterTypes);
                            int assignable2 = this.isAssignable(parameterTypes, array2);
                            if (assignable2 && assignable) {
                                assignable = !t2.isSynthetic();
                                assignable2 = !t.isSynthetic();
                            }
                            if (assignable2 == assignable) {
                                n = true;
                            }
                            else if (!(assignable == 0)) {
                                t = t2;
                                array2 = parameterTypes;
                                n = false;
                            }
                        }
                    }
                }
                if (t2.isVarArgs()) {
                    int j = parameterTypes.length - 1;
                    if (j <= this.args.length) {
                        Class[] array3 = new Class[this.args.length];
                        System.arraycopy(parameterTypes, 0, array3, 0, j);
                        if (j < this.args.length) {
                            Class<Object> clazz = parameterTypes[j].getComponentType();
                            if (clazz.isPrimitive()) {
                                clazz = PrimitiveWrapperMap.getType(clazz.getName());
                            }
                            for (; j < this.args.length; ++j) {
                                array3[j] = clazz;
                            }
                        }
                        hashMap.put(t2, array3);
                    }
                }
            }
        }
        for (T t3 : array) {
            Class[] array4 = (Class[])hashMap.get(t3);
            if (array4 != null && this.isAssignable(array4, this.args)) {
                if (t == null) {
                    t = t3;
                    array2 = array4;
                }
                else {
                    int assignable3 = this.isAssignable(array2, array4);
                    int assignable4 = this.isAssignable(array4, array2);
                    if (assignable4 && assignable3) {
                        assignable3 = !t3.isSynthetic();
                        assignable4 = !t.isSynthetic();
                    }
                    if (assignable4 == assignable3) {
                        if (array2 == hashMap.get(t)) {
                            n = true;
                        }
                    }
                    else if (!(assignable3 == 0)) {
                        t = t3;
                        array2 = array4;
                        n = false;
                    }
                }
            }
        }
        if (!(n == 0)) {
            throw new NoSuchMethodException("Ambiguous methods are found");
        }
        if (t == null) {
            throw new NoSuchMethodException("Method is not found");
        }
        return t;
    }
    
    private boolean isAssignable(Class<?>[] array, Class<?>[] array2) {
        int i = 0;
        while (i < this.args.length) {
            if (null != this.args[i] && !array[i].isAssignableFrom(array2[i])) {
                return false;
            }
            ++i;
        }
        return true;
    }
}
